"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const amqp_client_node_1 = require("amqp-client-node");
const routing_keys_name_1 = require("../utils/routing.keys.name");
const exchange_name_1 = require("../utils/exchange.name");
const event_name_1 = require("../utils/event.name");
const configurations_1 = require("../utils/configurations");
const events_1 = require("events");
const endpoint_queue_1 = require("../utils/endpoint.queue");
const resource_name_1 = require("../utils/resource.name");
const defaultOptionPub = {
    exchange: {
        durable: true,
        type: 'direct'
    }
};
const defaultOptionSub = {
    consumer: { noAck: true },
    exchange: {
        durable: true,
        type: 'direct'
    },
    queue: {
        durable: true
    },
    receiveFromYourself: false
};
const defaultOptionRpcServer = {
    consumer: { noAck: false },
    exchange: {
        durable: true,
        type: 'direct'
    },
    queue: {
        durable: true
    }
};
const defaultConnOpt = {
    retries: 0,
    interval: 1000
};
class RabbitMQClient extends events_1.EventEmitter {
    constructor(appName, connParams, connOptions) {
        super();
        this.appName = appName;
        this.defaultOptionRpcClient = {
            exchange: {
                durable: true,
                type: 'direct'
            },
            rcpTimeout: 5000
        };
        this.defaultConnConfig = {
            protocol: 'amqp',
            hostname: '127.0.0.1',
            port: 5672,
            username: 'guest',
            password: 'guest',
            vhost: 'ocariot'
        };
        this._connConfig = this.defaultConnConfig;
        if (typeof connParams === 'object') {
            this._connConfig = Object.assign(Object.assign({}, this._connConfig), connParams);
        }
        if (typeof connParams === 'string') {
            this._connConfig = connParams.concat('/').concat(configurations_1.Configurations.VHOST);
        }
        this._connOpt = Object.assign(Object.assign({}, defaultConnOpt), connOptions);
        if (connOptions) {
            if (connOptions.receiveFromYourself !== undefined)
                defaultOptionSub.receiveFromYourself = connOptions.receiveFromYourself;
            if (connOptions.rpcTimeout)
                this.defaultOptionRpcClient.rcpTimeout = connOptions.rpcTimeout;
        }
    }
    pubEventInitialization() {
        this._pubConnection.on('close_connection', () => {
            this.emit('pub_disconnected');
        });
        this._pubConnection.on('re_established_connection', () => {
            this.emit('pub_reconnected');
        });
        this._pubConnection.on('trying_connect', () => {
            this.emit('pub_trying_connection');
        });
        this._pubConnection.on('lost_connection', () => {
            this.emit('pub_lost_connection');
        });
        this._pubConnection.on('error_connection', (err) => {
            this.emit('pub_connection_error', err);
        });
    }
    subEventInitialization() {
        this._subConnection.on('close_connection', () => {
            this.emit('sub_disconnected');
        });
        this._subConnection.on('re_established_connection', () => {
            this.emit('sub_reconnected');
        });
        this._subConnection.on('trying_connect', () => {
            this.emit('sub_trying_connection');
        });
        this._subConnection.on('lost_connection', () => {
            this.emit('sub_lost_connection');
        });
        this._subConnection.on('error_connection', (err) => {
            this.emit('sub_connection_error', err);
        });
    }
    serverEventInitialization() {
        this._serverConnection.on('close_connection', () => {
            this.emit('rpc_server_disconnected');
        });
        this._serverConnection.on('re_established_connection', () => {
            this.emit('rpc_server_reconnected');
        });
        this._serverConnection.on('trying_connect', () => {
            this.emit('rpc_server_trying_connection');
        });
        this._serverConnection.on('lost_connection', () => {
            this.emit('rpc_server_lost_connection');
        });
        this._serverConnection.on('error_connection', (err) => {
            this.emit('rpc_server_connection_error', err);
        });
    }
    clientEventInitialization() {
        this._clientConnection.on('close_connection', () => {
            this.emit('rpc_client_disconnected');
        });
        this._clientConnection.on('re_established_connection', () => {
            this.emit('rpc_client_reconnected');
        });
        this._clientConnection.on('trying_connect', () => {
            this.emit('rpc_client_trying_connection');
        });
        this._clientConnection.on('lost_connection', () => {
            this.emit('rpc_client_lost_connection');
        });
        this._clientConnection.on('error_connection', (err) => {
            this.emit('rpc_client_connection_error', err);
        });
    }
    logger(level) {
        if (level === 'warn' || level === 'error' || level === 'info' || !level) {
            amqp_client_node_1.amqpClient.logger(level);
        }
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (this._pubConnection)
                    yield this._pubConnection.close();
                if (this._subConnection)
                    yield this._subConnection.close();
                if (this._clientConnection)
                    yield this._clientConnection.close();
                if (this._serverConnection)
                    yield this._serverConnection.close();
                return Promise.resolve();
            }
            catch (err) {
                return Promise.reject(err);
            }
        });
    }
    pubConnection() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!this._pubConnection && !this._pubConnectionInitialized) {
                    this._pubConnectionInitialized = amqp_client_node_1.amqpClient.createConnetion(this._connConfig, this._connOpt);
                    this._pubConnection = yield this._pubConnectionInitialized;
                    this.pubEventInitialization();
                    this.emit('pub_connected');
                }
                yield this._pubConnectionInitialized;
                return Promise.resolve();
            }
            catch (err) {
                this._pubConnection = undefined;
                this._pubConnectionInitialized = undefined;
                return Promise.reject(err);
            }
        });
    }
    publish(exchangeName, routingKey, body) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.pubConnection();
                const message = { content: body };
                return this._pubConnection.pub(exchangeName, routingKey, message, defaultOptionPub);
            }
            catch (err) {
                return Promise.reject(err);
            }
        });
    }
    pub(routingKey, body) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.pubConnection();
                const message = { content: body };
                return this._pubConnection.pub(exchange_name_1.ExchangeName.PERSONALIZED, routingKey, message, {
                    exchange: {
                        durable: true,
                        type: 'topic'
                    }
                });
            }
            catch (err) {
                return Promise.reject(err);
            }
        });
    }
    subConnection() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!this._subConnection && !this._subConnectionInitialized) {
                    this._subConnectionInitialized = amqp_client_node_1.amqpClient.createConnetion(this._connConfig, this._connOpt);
                    this._subConnection = yield this._subConnectionInitialized;
                    this.subEventInitialization();
                    this.emit('sub_connected');
                }
                yield this._subConnectionInitialized;
                return Promise.resolve();
            }
            catch (err) {
                this._subConnection = undefined;
                this._subConnectionInitialized = undefined;
                return Promise.reject(err);
            }
        });
    }
    sub(routingKey, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.subConnection();
                const options = Object.assign(Object.assign({}, defaultOptionSub), {
                    exchange: { durable: true, type: 'topic' }
                });
                return this._subConnection.sub(this.appName.concat(endpoint_queue_1.EndpointQueue.PERSONALIZED_SUB), exchange_name_1.ExchangeName.PERSONALIZED, routingKey, msg => callback(msg.content), options);
            }
            catch (err) {
                return Promise.reject(err);
            }
        });
    }
    subscribe(exchangeName, routingKey, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.subConnection();
                return this._subConnection.sub(this.appName, exchangeName, routingKey, msg => callback(msg.content), defaultOptionSub);
            }
            catch (err) {
                return Promise.reject(err);
            }
        });
    }
    serverConnection() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!this._serverConnectionInitialized) {
                    this._serverConnectionInitialized = amqp_client_node_1.amqpClient.createConnetion(this._connConfig, this._connOpt);
                    this._serverConnection = yield this._serverConnectionInitialized;
                    this.serverEventInitialization();
                    this.emit('rpc_server_connected');
                }
                yield this._serverConnectionInitialized;
                return Promise.resolve();
            }
            catch (err) {
                this._serverConnection = undefined;
                this._serverConnectionInitialized = undefined;
                return Promise.reject(err);
            }
        });
    }
    resource(exchangeName, name, func) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.serverConnection();
                const server = this._serverConnection
                    .createRpcServer(this.appName.concat(endpoint_queue_1.EndpointQueue.RPC), exchangeName, [], defaultOptionRpcServer);
                if (server.addResource(name, func))
                    return server.start();
                return Promise.reject(new Error('Could not add resource to RPC Server. Resource already exists!'));
            }
            catch (err) {
                return Promise.reject(err);
            }
        });
    }
    provide(name, func) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.serverConnection();
                const server = this._serverConnection
                    .createRpcServer(this.appName.concat(endpoint_queue_1.EndpointQueue.PERSONALIZED_RPC), exchange_name_1.ExchangeName.PERSONALIZED_RPC, [], defaultOptionRpcServer);
                if (server.addResource(name, func))
                    return server.start();
                return Promise.reject(new Error('Could not add resource to RPC Server. Resource already exists!'));
            }
            catch (err) {
                return Promise.reject(err);
            }
        });
    }
    getResource(name, params, callback) {
        if (!callback) {
            return this.getResourcePromise(exchange_name_1.ExchangeName.PERSONALIZED_RPC, name, params);
        }
        this.getResourceCallback(exchange_name_1.ExchangeName.PERSONALIZED_RPC, name, params, callback);
    }
    requestResource(exchangeName, name, params, callback) {
        if (!callback) {
            return this.getResourcePromise(exchangeName, name, params);
        }
        this.getResourceCallback(exchangeName, name, params, callback);
    }
    getResourceCallback(exchangeName, name, params, callback) {
        let time;
        if (!this._clientConnection) {
            new Promise((res) => {
                time = setTimeout(res, this.defaultOptionRpcClient.rcpTimeout);
            }).then(() => {
                return callback(new Error('rpc timed out'));
            });
        }
        if (!this._clientConnection && !this._clientConnectionInitialized) {
            this._clientConnectionInitialized = amqp_client_node_1.amqpClient.createConnetion(this._connConfig, this._connOpt);
            this._clientConnectionInitialized
                .then((conn) => {
                this._clientConnection = conn;
                this.clientEventInitialization();
                this.emit('rpc_client_connected');
            })
                .catch(() => {
                this._clientConnectionInitialized = undefined;
            });
        }
        this._clientConnectionInitialized
            .then(() => {
            clearTimeout(time);
            this._clientConnection.rpcClient(exchangeName, name, params, callback, this.defaultOptionRpcClient);
        })
            .catch(err => {
            callback(err, null);
        });
    }
    getResourcePromise(exchangeName, name, params) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    let time;
                    if (!this._clientConnection) {
                        new Promise((res) => {
                            time = setTimeout(res, this.defaultOptionRpcClient.rcpTimeout);
                        }).then(() => {
                            return reject(new Error('rpc timed out'));
                        });
                    }
                    if (!this._clientConnection && !this._clientConnectionInitialized) {
                        this._clientConnectionInitialized = amqp_client_node_1.amqpClient.createConnetion(this._connConfig, this._connOpt);
                        this._clientConnection = yield this._clientConnectionInitialized;
                        this.clientEventInitialization();
                        this.emit('rpc_client_connected');
                    }
                    yield this._clientConnectionInitialized;
                    clearTimeout(time);
                    return resolve(this._clientConnection.rpcClient(exchangeName, name, params, this.defaultOptionRpcClient));
                }
                catch (err) {
                    this._clientConnectionInitialized = undefined;
                    return reject(err);
                }
            }));
        });
    }
    pubSavePhysicalActivity(activity) {
        const message = {
            event_name: event_name_1.EventName.SAVE_PHYSICAL_ACTIVITY_EVENT,
            timestamp: new Date().toISOString(),
            physicalactivity: activity
        };
        return this.publish(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.SAVE_PHYSICAL_ACTIVITIES, message);
    }
    pubUpdatePhysicalActivity(activity) {
        const message = {
            event_name: event_name_1.EventName.UPDATE_PHYSICAL_ACTIVITY_EVENT,
            timestamp: new Date().toISOString(),
            physicalactivity: activity
        };
        return this.publish(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.UPDATE_PHYSICAL_ACTIVITIES, message);
    }
    pubDeletePhysicalActivity(activity) {
        const message = {
            event_name: event_name_1.EventName.DELETE_PHYSICAL_ACTIVITY_EVENT,
            timestamp: new Date().toISOString(),
            physicalactivity: activity
        };
        return this.publish(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.DELETE_PHYSICAL_ACTIVITIES, message);
    }
    pubSaveSleep(sleep) {
        const message = {
            event_name: event_name_1.EventName.SAVE_SLEEP_EVENT,
            timestamp: new Date().toISOString(),
            sleep
        };
        return this.publish(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.SAVE_SLEEP, message);
    }
    pubUpdateSleep(sleep) {
        const message = {
            event_name: event_name_1.EventName.UPDATE_SLEEP_EVENT,
            timestamp: new Date().toISOString(),
            sleep
        };
        return this.publish(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.UPDATE_SLEEP, message);
    }
    pubDeleteSleep(sleep) {
        const message = {
            event_name: event_name_1.EventName.DELETE_SLEEP_EVENT,
            timestamp: new Date().toISOString(),
            sleep
        };
        return this.publish(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.DELETE_SLEEP, message);
    }
    pubSaveWeight(weight) {
        const message = {
            event_name: event_name_1.EventName.SAVE_WEIGHT_EVENT,
            timestamp: new Date().toISOString(),
            weight
        };
        return this.publish(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.SAVE_WEIGHTS, message);
    }
    pubDeleteWeight(weight) {
        const message = {
            event_name: event_name_1.EventName.DELETE_WEIGHT_EVENT,
            timestamp: new Date().toISOString(),
            weight
        };
        return this.publish(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.DELETE_WEIGHTS, message);
    }
    pubSaveEnvironment(environment) {
        const message = {
            event_name: event_name_1.EventName.SAVE_ENVIRONMENT_EVENT,
            timestamp: new Date().toISOString(),
            environment
        };
        return this.publish(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.SAVE_ENVIRONMENTS, message);
    }
    pubDeleteEnvironment(environment) {
        const message = {
            event_name: event_name_1.EventName.DELETE_ENVIRONMENT_EVENT,
            timestamp: new Date().toISOString(),
            environment
        };
        return this.publish(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.DELETE_ENVIRONMENTS, message);
    }
    pubSaveLog(log) {
        const message = {
            event_name: event_name_1.EventName.SAVE_LOG_EVENT,
            timestamp: new Date().toISOString(),
            log
        };
        return this.publish(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.SAVE_LOG, message);
    }
    pubSaveChild(child) {
        const message = {
            event_name: event_name_1.EventName.SAVE_CHILD_EVENT,
            timestamp: new Date().toISOString(),
            child
        };
        return this.publish(exchange_name_1.ExchangeName.ACCOUNT, routing_keys_name_1.RoutingKeysName.UPDATE_CHILDREN, message);
    }
    pubUpdateChild(child) {
        const message = {
            event_name: event_name_1.EventName.UPDATE_CHILD_EVENT,
            timestamp: new Date().toISOString(),
            child
        };
        return this.publish(exchange_name_1.ExchangeName.ACCOUNT, routing_keys_name_1.RoutingKeysName.UPDATE_CHILDREN, message);
    }
    pubUpdateFamily(family) {
        const message = {
            event_name: event_name_1.EventName.UPDATE_FAMILY_EVENT,
            timestamp: new Date().toISOString(),
            family
        };
        return this.publish(exchange_name_1.ExchangeName.ACCOUNT, routing_keys_name_1.RoutingKeysName.UPDATE_FAMILIES, message);
    }
    pubUpdateEducator(educator) {
        const message = {
            event_name: event_name_1.EventName.UPDATE_EDUCATOR_EVENT,
            timestamp: new Date().toISOString(),
            educator
        };
        return this.publish(exchange_name_1.ExchangeName.ACCOUNT, routing_keys_name_1.RoutingKeysName.UPDATE_EDUCATORS, message);
    }
    pubUpdateHealthProfessional(healthprofessional) {
        const message = {
            event_name: event_name_1.EventName.UPDATE_HEALTH_PROFESSIONAL_EVENT,
            timestamp: new Date().toISOString(),
            healthprofessional
        };
        return this.publish(exchange_name_1.ExchangeName.ACCOUNT, routing_keys_name_1.RoutingKeysName.UPDATE_HEALTH_PROFESSIONALS, message);
    }
    pubUpdateApplication(application) {
        const message = {
            event_name: event_name_1.EventName.UPDATE_APPLICATION_EVENT,
            timestamp: new Date().toISOString(),
            application
        };
        return this.publish(exchange_name_1.ExchangeName.ACCOUNT, routing_keys_name_1.RoutingKeysName.UPDATE_APPLICATIONS, message);
    }
    pubDeleteUser(user) {
        const message = {
            event_name: event_name_1.EventName.DELETE_USER_EVENT,
            timestamp: new Date().toISOString(),
            user
        };
        return this.publish(exchange_name_1.ExchangeName.ACCOUNT, routing_keys_name_1.RoutingKeysName.DELETE_USERS, message);
    }
    pubDeleteInstitution(institution) {
        const message = {
            event_name: event_name_1.EventName.DELETE_INSTITUTION_EVENT,
            timestamp: new Date().toISOString(),
            institution
        };
        return this.publish(exchange_name_1.ExchangeName.ACCOUNT, routing_keys_name_1.RoutingKeysName.DELETE_INSTITUTIONS, message);
    }
    pubFitbitLastSync(datetime) {
        const message = {
            event_name: event_name_1.EventName.LASTSYNC_FIIBIT_FITBIT,
            timestamp: new Date().toISOString(),
            datetime
        };
        return this.publish(exchange_name_1.ExchangeName.DATA_SYNC, routing_keys_name_1.RoutingKeysName.LAST_SYNC_FIIBIT, message);
    }
    pubFitbitAuthError(error) {
        const message = {
            event_name: event_name_1.EventName.ERROR_FITBIT_AUTH_EVENT,
            timestamp: new Date().toISOString(),
            error
        };
        return this.publish(exchange_name_1.ExchangeName.DATA_SYNC, routing_keys_name_1.RoutingKeysName.ERROR_FITBIT_AUTH, message);
    }
    subSavePhysicalActivity(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.SAVE_PHYSICAL_ACTIVITIES, callback);
    }
    subUpdatePhysicalActivity(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.UPDATE_PHYSICAL_ACTIVITIES, callback);
    }
    subDeletePhysicalActivity(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.DELETE_PHYSICAL_ACTIVITIES, callback);
    }
    subSaveSleep(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.SAVE_SLEEP, callback);
    }
    subUpdateSleep(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.UPDATE_SLEEP, callback);
    }
    subDeleteSleep(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.DELETE_SLEEP, callback);
    }
    subSaveWeight(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.SAVE_WEIGHTS, callback);
    }
    subDeleteWeight(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.DELETE_WEIGHTS, callback);
    }
    subSaveEnvironment(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.SAVE_ENVIRONMENTS, callback);
    }
    subDeleteEnvironment(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.DELETE_ENVIRONMENTS, callback);
    }
    subSaveLog(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.ACTIVITY_TRACKING, routing_keys_name_1.RoutingKeysName.SAVE_LOG, callback);
    }
    subSaveChild(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.ACCOUNT, routing_keys_name_1.RoutingKeysName.SAVE_CHILDREN, callback);
    }
    subUpdateChild(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.ACCOUNT, routing_keys_name_1.RoutingKeysName.UPDATE_CHILDREN, callback);
    }
    subUpdateFamily(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.ACCOUNT, routing_keys_name_1.RoutingKeysName.UPDATE_FAMILIES, callback);
    }
    subUpdateEducator(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.ACCOUNT, routing_keys_name_1.RoutingKeysName.UPDATE_EDUCATORS, callback);
    }
    subUpdateHealthProfessional(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.ACCOUNT, routing_keys_name_1.RoutingKeysName.UPDATE_HEALTH_PROFESSIONALS, callback);
    }
    subUpdateApplication(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.ACCOUNT, routing_keys_name_1.RoutingKeysName.UPDATE_APPLICATIONS, callback);
    }
    subDeleteUser(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.ACCOUNT, routing_keys_name_1.RoutingKeysName.DELETE_USERS, callback);
    }
    subDeleteInstitution(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.ACCOUNT, routing_keys_name_1.RoutingKeysName.DELETE_INSTITUTIONS, callback);
    }
    subFitbitLastSync(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.DATA_SYNC, routing_keys_name_1.RoutingKeysName.LAST_SYNC_FIIBIT, callback);
    }
    subFitbitAuthError(callback) {
        return this.subscribe(exchange_name_1.ExchangeName.DATA_SYNC, routing_keys_name_1.RoutingKeysName.ERROR_FITBIT_AUTH, callback);
    }
    providePhysicalActivities(listener) {
        return this.resource(exchange_name_1.ExchangeName.ACTIVITY_TRACKING_RPC, resource_name_1.ResourceName.PHYSICAL_ACTIVITIES, listener);
    }
    provideSleep(listener) {
        return this.resource(exchange_name_1.ExchangeName.ACTIVITY_TRACKING_RPC, resource_name_1.ResourceName.SLEEP, listener);
    }
    provideWeights(listener) {
        return this.resource(exchange_name_1.ExchangeName.ACTIVITY_TRACKING_RPC, resource_name_1.ResourceName.WEIGHTS, listener);
    }
    provideEnvironments(listener) {
        return this.resource(exchange_name_1.ExchangeName.ACTIVITY_TRACKING_RPC, resource_name_1.ResourceName.ENVIRONMENTS, listener);
    }
    provideLogs(listener) {
        return this.resource(exchange_name_1.ExchangeName.ACTIVITY_TRACKING_RPC, resource_name_1.ResourceName.LOGS, listener);
    }
    provideChildren(listener) {
        return this.resource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.CHILDREN, listener);
    }
    provideFamilies(listener) {
        return this.resource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.FAMILIES, listener);
    }
    provideFamilyChildren(listener) {
        return this.resource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.FAMILY_CHILDREN, listener);
    }
    provideEducators(listener) {
        return this.resource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.EDUCATORS, listener);
    }
    provideEducatorChildrenGroups(listener) {
        return this.resource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.EDUCATORS_CHILDRES_GROUPS, listener);
    }
    provideHealthProfessionals(listener) {
        return this.resource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.HEALTH_PROFESSIONALS, listener);
    }
    provideHealthProfessionalChildrenGroups(listener) {
        return this.resource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.HEALTH_PROFESSIONAL_CHILDREN_GROUPS, listener);
    }
    provideApplications(listener) {
        return this.resource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.APPLICATIONS, listener);
    }
    provideInstitutions(listener) {
        return this.resource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.INSTITUTIONS, listener);
    }
    getPhysicalActivities(query, callback) {
        if (!callback) {
            return this.requestResource(exchange_name_1.ExchangeName.ACTIVITY_TRACKING_RPC, resource_name_1.ResourceName.PHYSICAL_ACTIVITIES, [query]);
        }
        this.requestResource(exchange_name_1.ExchangeName.ACTIVITY_TRACKING_RPC, resource_name_1.ResourceName.PHYSICAL_ACTIVITIES, [query], callback);
    }
    getSleep(query, callback) {
        if (!callback) {
            return this.requestResource(exchange_name_1.ExchangeName.ACTIVITY_TRACKING_RPC, resource_name_1.ResourceName.SLEEP, [query]);
        }
        this.requestResource(exchange_name_1.ExchangeName.ACTIVITY_TRACKING_RPC, resource_name_1.ResourceName.SLEEP, [query], callback);
    }
    getWeights(query, callback) {
        if (!callback) {
            return this.requestResource(exchange_name_1.ExchangeName.ACTIVITY_TRACKING_RPC, resource_name_1.ResourceName.WEIGHTS, [query]);
        }
        this.requestResource(exchange_name_1.ExchangeName.ACTIVITY_TRACKING_RPC, resource_name_1.ResourceName.WEIGHTS, [query], callback);
    }
    getEnvironments(query, callback) {
        if (!callback) {
            return this.requestResource(exchange_name_1.ExchangeName.ACTIVITY_TRACKING_RPC, resource_name_1.ResourceName.ENVIRONMENTS, [query]);
        }
        this.requestResource(exchange_name_1.ExchangeName.ACTIVITY_TRACKING_RPC, resource_name_1.ResourceName.ENVIRONMENTS, [query], callback);
    }
    getLogs(query, callback) {
        if (!callback) {
            return this.requestResource(exchange_name_1.ExchangeName.ACTIVITY_TRACKING_RPC, resource_name_1.ResourceName.LOGS, [query]);
        }
        this.requestResource(exchange_name_1.ExchangeName.ACTIVITY_TRACKING_RPC, resource_name_1.ResourceName.LOGS, [query], callback);
    }
    getChildren(query, callback) {
        if (!callback) {
            return this.requestResource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.CHILDREN, [query]);
        }
        this.requestResource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.CHILDREN, [query], callback);
    }
    getFamilies(query, callback) {
        if (!callback) {
            return this.requestResource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.FAMILIES, [query]);
        }
        this.requestResource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.FAMILIES, [query], callback);
    }
    getFamilyChildren(familyId, callback) {
        if (!callback) {
            return this.requestResource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.FAMILY_CHILDREN, [familyId]);
        }
        this.requestResource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.FAMILY_CHILDREN, [familyId], callback);
    }
    getEducators(query, callback) {
        if (!callback) {
            return this.requestResource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.EDUCATORS, [query]);
        }
        this.requestResource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.EDUCATORS, [query], callback);
    }
    getEducatorChildrenGroups(educatorId, callback) {
        if (!callback) {
            return this.requestResource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.EDUCATORS_CHILDRES_GROUPS, [educatorId]);
        }
        this.requestResource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.EDUCATORS_CHILDRES_GROUPS, [educatorId], callback);
    }
    getHealthProfessionals(query, callback) {
        if (!callback) {
            return this.requestResource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.HEALTH_PROFESSIONALS, [query]);
        }
        this.requestResource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.HEALTH_PROFESSIONALS, [query], callback);
    }
    getHealthProfessionalChildrenGroups(healthProfessionalId, callback) {
        if (!callback) {
            return this.requestResource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.HEALTH_PROFESSIONAL_CHILDREN_GROUPS, [healthProfessionalId]);
        }
        this.requestResource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.HEALTH_PROFESSIONAL_CHILDREN_GROUPS, [healthProfessionalId], callback);
    }
    getApplications(query, callback) {
        if (!callback) {
            return this.requestResource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.APPLICATIONS, [query]);
        }
        this.requestResource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.APPLICATIONS, [query], callback);
    }
    getInstitutions(query, callback) {
        if (!callback) {
            return this.requestResource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.INSTITUTIONS, [query]);
        }
        this.requestResource(exchange_name_1.ExchangeName.ACCOUNT_RPC, resource_name_1.ResourceName.INSTITUTIONS, [query], callback);
    }
}
exports.RabbitMQClient = RabbitMQClient;
